#Requires AutoHotkey v2.0+
#SingleInstance Force

; ----------------------------------------
; User define
; ----------------------------------------

; 再押下ごとの画面サイズを百分率で指定する
; Alt + [1]
LEFT_WIDTH := [0.5, 0.6, 0.7, 0.3, 0.4]
; Alt + [2]
CENTER_WIDTH := [0.5, 0.6, 0.7, 0.3, 0.4]
; Alt + [3]
RIGHT_WIDTH := [0.5, 0.6, 0.7, 0.3, 0.4]
; Alt + [qwe]
UPPER_HEIGHT := [0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 0.3, 0.4]
; Alt + [asd]
LOWER_HEIGHT := [0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 0.3, 0.4]

; ----------------------------------------
; Script define
; ----------------------------------------

; デスクトップ情報
DESKTOP_CLASS := "ahk_class Progman"
DESKTOP_CLASS2 := "ahk_class WorkerW"
DESKTOP_EXE := "ahk_exe explorer.exe"

; グローバル変数
previousInfo := ["", "", ""]

; ----------------------------------------
; helper function
; ----------------------------------------

; OS情報取得
getWinOSVersion()
{
    osVersion := StrSplit(A_OSVersion, ".")
    major := osVersion[1]
    minor := osVersion[2]
    build := osVersion[3]
    if (major == "10")
        if (minor == "0")
            if (build >= "20000")
                return 11
    return 10
}

; アクティブ画面のモニター番号取得
getMonitorNumberOfActiveWindow()
{
    WinGetPos(&x, &y, , , "A")
    monitorCount := MonitorGetCount()
    Loop monitorCount {
        MonitorGet(A_Index, &left, &top, &right, &bottom)
        if (x >= left
            and x <= right
            and y >= top
            and y <= bottom) {
            return A_Index
        }
    }
    return 1
}

; ディスプレイ情報取得
getDisplayInfo(monitorNum, &x, &y, &width, &height)
{
    if (monitorNum.Length > 0) {
        displayNum := monitorNum[1]
    } else {
        displayNum := getMonitorNumberOfActiveWindow()
    }
    MonitorGetWorkArea(displayNum, &left, &top, &right, &bottom)
    x := left
    y := top
    width := right - left
    height := bottom - top
}

; 画面サイズMAXの要素位置取得
getMaxValueIndex(array)
{
    index := 0
    value := 0
    Loop array.Length {
        if (array[A_Index] > value) {
            index := A_Index
            value := array[A_Index]
        }
    }
    return index
}

; 再押下ごとの要素位置取得
getRotateIndex(activeHwnd, windowPosition, array, direction*)
{
    previousHwnd := previousInfo[1]
    previousPosition := previousInfo[2]
    previousIndex := previousInfo[3]
    if ((previousHwnd == activeHwnd) and
        (previousPosition == windowPosition)) {
        forward := true
        if (direction.Length > 0) {
            if (GetKeyState("Shift") or (direction[1] == "reverse"))
                forward := false
            if (GetKeyState("Shift") and (direction[1] == "reverse"))
                forward := true
        } else {
            if (GetKeyState("Shift"))
                forward := false
        }
        if (forward) {
            if (previousIndex != array.Length)
                return previousIndex + 1
            else
                return 1
        } else {
            if (previousIndex == 1)
                return array.Length
            else
                return previousIndex - 1
        }
    }
    return 1
}

; 画面位置取得
getWindowPosition(monitorNum, activeHwnd, windowPosition, &x, &y, &width, &height)
{
    global previousInfo

    ; ディスプレイ情報取得
    getDisplayInfo(monitorNum, &x, &y, &width, &height)

    ; 画面サイズ補正
    ; 画面の左,右,下に隙間が空くので補正する。
    ; ウインドスタイル「WS_SYSMENU」が設定されていない画面は補正なし。(例：VSCode)
    edge := 0
    if (getWinOSVersion() == 11) {
        style := WinGetStyle("A")
        if (style & 0x00080000) { ; WS_SYSMENU
            edge := 7 ; px
        }
    }

    ; 画面位置計算
    if (windowPosition == "1") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, LEFT_WIDTH)
        x := x - edge
        y := y
        width := width * LEFT_WIDTH[rotateIndex] + edge * 2
        height := height + edge
    }
    if (windowPosition == "2") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, CENTER_WIDTH)
        x := width * (1 - CENTER_WIDTH[rotateIndex]) / 2 - edge
        y := y
        width := width * CENTER_WIDTH[rotateIndex] + edge * 2
        height := height + edge
    }
    if (windowPosition == "3") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, RIGHT_WIDTH)
        x := width * (1 - RIGHT_WIDTH[rotateIndex]) - edge
        y := y
        width := width * RIGHT_WIDTH[rotateIndex] + edge * 2
        height := height + edge
    }
    if (windowPosition == "q") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, LOWER_HEIGHT)
        x := x - edge
        y := y
        width := width / 2 + edge * 2
        height := height * LOWER_HEIGHT[rotateIndex] + edge
    }
    if (windowPosition == "a") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, LOWER_HEIGHT)
        x := x - edge
        y := height * (1 - LOWER_HEIGHT[rotateIndex]) + y
        width := width / 2 + edge * 2
        height := height * LOWER_HEIGHT[rotateIndex] + edge
    }
    if (windowPosition == "w") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, UPPER_HEIGHT)
        x := width / 2 - edge
        y := y
        width := width / 2 + edge * 2
        height := height * UPPER_HEIGHT[rotateIndex] + edge
    }
    if (windowPosition == "s") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, LOWER_HEIGHT)
        x := width / 2 - edge
        y := height * (1 - LOWER_HEIGHT[rotateIndex]) + y
        width := width / 2 + edge * 2
        height := height * LOWER_HEIGHT[rotateIndex] + edge
    }
    if (windowPosition == "e") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, UPPER_HEIGHT)
        x := x - edge
        y := y
        width := width + edge * 2
        height := height * UPPER_HEIGHT[rotateIndex] + edge
    }
    if (windowPosition == "d") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, LOWER_HEIGHT)
        x := x - edge
        y := height * (1 - LOWER_HEIGHT[rotateIndex]) + y
        width := width + edge * 2
        height := height * LOWER_HEIGHT[rotateIndex] + edge
    }
    ; 矢印キー（上/下）
    if (windowPosition == "Up") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, CENTER_WIDTH)
        x := width * (1 - CENTER_WIDTH[rotateIndex]) / 2 - edge
        y := y
        width := width * CENTER_WIDTH[rotateIndex] + edge * 2
        height := height + edge
    }
    if (windowPosition == "Down") {
        rotateIndex := getRotateIndex(activeHwnd, windowPosition, CENTER_WIDTH, "reverse")
        x := width * (1 - CENTER_WIDTH[rotateIndex]) / 2 - edge
        y := y
        width := width * CENTER_WIDTH[rotateIndex] + edge * 2
        height := height + edge
    }
    ; 矢印キー（左/右）
    previousPosition := previousInfo[2]
    previousIndex := previousInfo[3]
    if (windowPosition == "Left") {
        if ((previousPosition == "Right") and
            (previousIndex != getMaxValueIndex(RIGHT_WIDTH))) {
            windowPosition := "Right"
            rotateIndex := getRotateIndex(activeHwnd, windowPosition, RIGHT_WIDTH)
            x := width * (1 - RIGHT_WIDTH[rotateIndex]) - edge
            y := y
            width := width * RIGHT_WIDTH[rotateIndex] + edge * 2
            height := height + edge
        } else {
            rotateIndex := getRotateIndex(activeHwnd, windowPosition, LEFT_WIDTH, "reverse")
            x := x - edge
            y := y
            width := width * LEFT_WIDTH[rotateIndex] + edge * 2
            height := height + edge
        }
    } else if (windowPosition == "Right") {
        if ((previousPosition == "Left") and
            (previousIndex != getMaxValueIndex(LEFT_WIDTH))) {
            windowPosition := "Left"
            rotateIndex := getRotateIndex(activeHwnd, windowPosition, LEFT_WIDTH)
            x := x - edge
            y := y
            width := width * LEFT_WIDTH[rotateIndex] + edge * 2
            height := height + edge
        } else {
            rotateIndex := getRotateIndex(activeHwnd, windowPosition, RIGHT_WIDTH, "reverse")
            x := width * (1 - RIGHT_WIDTH[rotateIndex]) - edge
            y := y
            width := width * RIGHT_WIDTH[rotateIndex] + edge * 2
            height := height + edge
        }
    }
    OutputDebug("x=" x " y=" y " width=" width " height=" height " windowPosition=" windowPosition " rotateIndex=" rotateIndex)

    ; 前回の移動先保存
    previousInfo[1] := activeHwnd
    previousInfo[2] := windowPosition
    previousInfo[3] := rotateIndex
}

; 画面位置設定
setWindowPosition(windowPosition, monitorNum*)
{
    try {
        ; デスクトップのアイコンが移動してしまうのでガード
        activeHwnd := WinGetID("A")
        GroupAdd("Desktop", DESKTOP_CLASS " " DESKTOP_EXE)
        GroupAdd("Desktop", DESKTOP_CLASS2 " " DESKTOP_EXE)
        desktopHwnd := WinGetID("ahk_group Desktop")
        if (activeHwnd == desktopHwnd)
            return

        ; 画面の最大化を解除
        if (WinGetMinMax(activeHwnd) == 1)
            WinRestore(activeHwnd)

        ; 画面サイズ変更の順序
        windowPosition := StrReplace(windowPosition, "+")

        ; 移動位置取得
        getWindowPosition(monitorNum, activeHwnd, windowPosition, &x, &y, &width, &height)

        ; 画面移動
        WinMove(x, y, width, height, activeHwnd)

    }
}

; ----------------------------------------
; main
; ----------------------------------------

; リマッピング
; https://ahkscript.github.io/ja/docs/v2/misc/Remap.htm#Remap
; ホットキーモディファイアの記号
; https://ahkscript.github.io/ja/docs/v2/Hotkeys.htm#Symbols
; キーリスト
; https://ahkscript.github.io/ja/docs/v2/KeyList.htm#keyboard

; 矢印キー × 5
^Up:: Send "{Up 5}"
^Down:: Send "{Down 5}"
^Left:: Send "{Left 5}"
^Right:: Send "{Right 5}"

; 上下左右
^*i:: Send "{Up}"
^*k:: Send "{Down}"
^*j:: Send "{Left}"
^*l:: Send "{Right}"

; Home/End
^*u:: Send "{Home}"
^*o:: Send "{End}"

; 削除
^*h:: Send "{BS}"

; 音量調整
^F10:: Send "{VOLUME_MUTE}"
^F11:: Send "{VOLUME_DOWN}"
^F12:: Send "{VOLUME_UP}"

; HHKB
; !Left:: Send "{Home}"
; !Right:: Send "{End}"
; !+Left:: Send "+{Home}"
; !+Right:: Send "+{End}"


; 画面移動
; setWindowPosition(windowPosition, monitorNum*)
;   windowPosition：[1|2|3|q|a|w|s|e|d]
;   monitorNum：[1-9]
;   *：省略可能

; 左/中央/右
!*1:: setWindowPosition("1")
!*2:: setWindowPosition("2")
!*3:: setWindowPosition("3")

; 左/中央/右
!*Left:: setWindowPosition("Left")
!*Up:: setWindowPosition("Up")
!*Down:: setWindowPosition("Down")
!*Right:: setWindowPosition("Right")

; 左上/左下/右上/右下
!*q:: setWindowPosition("q")
!*a:: setWindowPosition("a")
!*w:: setWindowPosition("w")
!*s:: setWindowPosition("s")

; 上/下
!*e:: setWindowPosition("e")
!*d:: setWindowPosition("d")


; 仮想デスクトップ移動
#Include "*i C:\tool\taskview\taskview_ext.ahk"